@@ -94,7 +94,8 @@ typedef enum {
  */
 #define GST_DEBUG_CAPS(string, caps) \
   GST_DEBUG ( string "%s: " GST_PTR_FORMAT, caps)
-#endif
+
+#endif /* GST_DISABLE_DEPRECATED */
 
 /**
  * GST_STATIC_CAPS:
@@ -105,8 +106,9 @@ typedef enum {
  */
 #define GST_STATIC_CAPS(string) \
 { \
-  /* caps */ { 0 }, \
+  /* caps */ { 0, 0, (GstCapsFlags) 0, NULL, GST_PADDING_INIT }, \
   /* string */ string, \
+  GST_PADDING_INIT \
 }
 
 typedef struct _GstCaps GstCaps;
@@ -171,77 +173,81 @@ struct _GstStaticCaps {
   gpointer _gst_reserved[GST_PADDING];
 };
 
-GType                    gst_caps_get_type                              (void);
-GstCaps *                gst_caps_new_empty                             (void);
-GstCaps *                gst_caps_new_any                               (void);
-GstCaps *                gst_caps_new_simple                            (const char    *media_type,
-									 const char    *fieldname,
-									 ...);
-GstCaps *                gst_caps_new_full                              (GstStructure  *struct1,
-									                 ...);
-GstCaps *                gst_caps_new_full_valist                       (GstStructure  *structure,
-									 va_list        var_args);
+GType             gst_caps_get_type                (void);
+GstCaps *         gst_caps_new_empty               (void);
+GstCaps *         gst_caps_new_any                 (void);
+GstCaps *         gst_caps_new_simple              (const char    *media_type,
+                                                    const char    *fieldname,
+                                                    ...);
+GstCaps *         gst_caps_new_full                (GstStructure  *struct1, ...);
+GstCaps *         gst_caps_new_full_valist         (GstStructure  *structure,
+                                                    va_list        var_args);
 
 /* reference counting */
-GstCaps *                gst_caps_ref					(GstCaps* caps);
-GstCaps *                gst_caps_copy					(const GstCaps * caps);
-GstCaps *                gst_caps_make_writable				(GstCaps *caps);
-void                     gst_caps_unref					(GstCaps* caps);
+GstCaps *         gst_caps_ref                     (GstCaps       *caps);
+GstCaps *         gst_caps_copy                    (const GstCaps *caps);
+GstCaps *         gst_caps_make_writable           (GstCaps       *caps);
+void              gst_caps_unref                   (GstCaps       *caps);
 
-GType			 gst_static_caps_get_type			(void);
-GstCaps *		 gst_static_caps_get                            (GstStaticCaps *static_caps);
+GType             gst_static_caps_get_type         (void);
+GstCaps *         gst_static_caps_get              (GstStaticCaps *static_caps);
 
 /* manipulation */
-void                     gst_caps_append                                (GstCaps       *caps1,
-									 GstCaps       *caps2);
-void                     gst_caps_append_structure                      (GstCaps       *caps,
-									 GstStructure  *structure);
-guint                    gst_caps_get_size                              (const GstCaps *caps);
-GstStructure *           gst_caps_get_structure                         (const GstCaps *caps,
-									 guint          index);
-GstCaps *		 gst_caps_copy_nth                              (const GstCaps * caps, guint nth);
-void			 gst_caps_truncate                              (GstCaps * caps);
-void                     gst_caps_set_simple                            (GstCaps       *caps,
-									 char          *field, ...) G_GNUC_NULL_TERMINATED;
-void                     gst_caps_set_simple_valist                     (GstCaps       *caps,
-									 char          *field,
-									 va_list        varargs);
+void              gst_caps_append                  (GstCaps       *caps1,
+                                                    GstCaps       *caps2);
+void              gst_caps_merge                   (GstCaps       *caps1,
+                                                    GstCaps       *caps2);
+void              gst_caps_append_structure        (GstCaps       *caps,
+                                                    GstStructure  *structure);
+void              gst_caps_remove_structure        (GstCaps       *caps, guint idx);
+void              gst_caps_merge_structure         (GstCaps       *caps,
+                                                    GstStructure  *structure);
+guint             gst_caps_get_size                (const GstCaps *caps);
+GstStructure *    gst_caps_get_structure           (const GstCaps *caps,
+                                                    guint          index);
+GstCaps *         gst_caps_copy_nth                (const GstCaps *caps, guint nth);
+void              gst_caps_truncate                (GstCaps       *caps);
+void              gst_caps_set_simple              (GstCaps       *caps,
+                                                    char          *field, ...) G_GNUC_NULL_TERMINATED;
+void              gst_caps_set_simple_valist       (GstCaps       *caps,
+                                                    char          *field,
+                                                    va_list        varargs);
 
 /* tests */
-gboolean                 gst_caps_is_any                                (const GstCaps *caps);
-gboolean                 gst_caps_is_empty                              (const GstCaps *caps);
-gboolean                 gst_caps_is_fixed                              (const GstCaps *caps);
-gboolean                 gst_caps_is_always_compatible                  (const GstCaps *caps1,
-									 const GstCaps *caps2);
-gboolean		 gst_caps_is_subset				(const GstCaps *subset,
-									 const GstCaps *superset);
-gboolean		 gst_caps_is_equal				(const GstCaps *caps1,
-									 const GstCaps *caps2);
-gboolean		 gst_caps_is_equal_fixed			(const GstCaps * caps1,
-									 const GstCaps * caps2);
+gboolean          gst_caps_is_any                  (const GstCaps *caps);
+gboolean          gst_caps_is_empty                (const GstCaps *caps);
+gboolean          gst_caps_is_fixed                (const GstCaps *caps);
+gboolean          gst_caps_is_always_compatible    (const GstCaps *caps1,
+                                                    const GstCaps *caps2);
+gboolean          gst_caps_is_subset		   (const GstCaps *subset,
+						    const GstCaps *superset);
+gboolean          gst_caps_is_equal		   (const GstCaps *caps1,
+						    const GstCaps *caps2);
+gboolean          gst_caps_is_equal_fixed          (const GstCaps *caps1,
+						    const GstCaps *caps2);
 
 
 /* operations */
-GstCaps *                gst_caps_intersect                             (const GstCaps *caps1,
-									 const GstCaps *caps2);
-GstCaps *                gst_caps_subtract				(const GstCaps *minuend,
-									 const GstCaps *subtrahend);
-GstCaps *                gst_caps_union                                 (const GstCaps *caps1,
-									 const GstCaps *caps2);
-GstCaps *                gst_caps_normalize                             (const GstCaps *caps);
-gboolean                 gst_caps_do_simplify                           (GstCaps *caps);
+GstCaps *         gst_caps_intersect               (const GstCaps *caps1,
+						    const GstCaps *caps2);
+GstCaps *         gst_caps_subtract		   (const GstCaps *minuend,
+						    const GstCaps *subtrahend);
+GstCaps *         gst_caps_union                   (const GstCaps *caps1,
+						    const GstCaps *caps2);
+GstCaps *         gst_caps_normalize               (const GstCaps *caps);
+gboolean          gst_caps_do_simplify             (GstCaps       *caps);
 
 #ifndef GST_DISABLE_LOADSAVE
-xmlNodePtr               gst_caps_save_thyself                          (const GstCaps *caps,
-									 xmlNodePtr     parent);
-GstCaps *                gst_caps_load_thyself                          (xmlNodePtr     parent);
+xmlNodePtr        gst_caps_save_thyself            (const GstCaps *caps,
+                                                    xmlNodePtr     parent);
+GstCaps *         gst_caps_load_thyself            (xmlNodePtr     parent);
 #endif
 
 /* utility */
-void                     gst_caps_replace                               (GstCaps      **caps,
-									 GstCaps       *newcaps);
-gchar *                  gst_caps_to_string                             (const GstCaps *caps);
-GstCaps *                gst_caps_from_string                           (const gchar   *string);
+void              gst_caps_replace                 (GstCaps      **caps,
+                                                    GstCaps       *newcaps);
+gchar *           gst_caps_to_string               (const GstCaps *caps);
+GstCaps *         gst_caps_from_string             (const gchar   *string);
 
 G_END_DECLS
 
