@@ -33,9 +33,9 @@
 #include "config.h"
 #endif
 
+#include "gst/gst_private.h"
 #include "gst/gstminiobject.h"
 #include "gst/gstinfo.h"
-#include "gst/gst_private.h"
 #include <gobject/gvaluecollector.h>
 
 #ifndef GST_DISABLE_TRACE
@@ -187,6 +187,20 @@ gst_mini_object_new (GType type)
   return mini_object;
 }
 
+/* FIXME 0.11: Current way of doing the copy makes it impossible
+ * to currectly chain to the parent classes and do a copy in a
+ * subclass without knowing all internals of the parent classes.
+ *
+ * For 0.11 we should do something like the following:
+ *  - The GstMiniObjectClass::copy() implementation of GstMiniObject
+ *    should call g_type_create_instance() with the type of the source
+ *    object.
+ *  - All GstMiniObjectClass::copy() implementations should as first
+ *    thing chain up to the parent class and then do whatever they need
+ *    to do to copy their type specific data. Note that this way the
+ *    instance_init() functions are called!
+ */
+
 /**
  * gst_mini_object_copy:
  * @mini_object: the mini-object to copy
@@ -364,13 +378,18 @@ gst_mini_object_replace (GstMiniObject ** olddata, GstMiniObject * newdata)
       newdata, newdata ? newdata->refcount : 0);
 #endif
 
+  olddata_val = g_atomic_pointer_get ((gpointer *) olddata);
+
+  if (olddata_val == newdata)
+    return;
+
   if (newdata)
     gst_mini_object_ref (newdata);
 
-  do {
+  while (!g_atomic_pointer_compare_and_exchange ((gpointer *) olddata,
+          olddata_val, newdata)) {
     olddata_val = g_atomic_pointer_get ((gpointer *) olddata);
-  } while (!g_atomic_pointer_compare_and_exchange ((gpointer *) olddata,
-          olddata_val, newdata));
+  }
 
   if (olddata_val)
     gst_mini_object_unref (olddata_val);
