@@ -59,7 +59,7 @@ static gboolean
 gst_registry_save (GstRegistry * registry, gchar * format, ...)
 {
   va_list var_args;
-  size_t written, len;
+  gsize written, len;
   gboolean ret;
   char *str;
 
@@ -123,15 +123,35 @@ read_string (xmlTextReaderPtr reader, gchar ** write_to, gboolean allow_blank)
       return found;
     }
     if (xmlTextReaderNodeType (reader) == XML_READER_TYPE_TEXT) {
+      xmlChar *value;
+
       if (found)
         return FALSE;
-      *write_to = (gchar *) xmlTextReaderValue (reader);
+
+      value = xmlTextReaderValue (reader);
+      *write_to = g_strdup ((gchar *) value);
+      xmlFree (value);
+
       found = TRUE;
     }
   }
   return FALSE;
 }
 
+static gboolean
+read_const_interned_string (xmlTextReaderPtr reader, const gchar ** write_to,
+    gboolean allow_blank)
+{
+  gchar *s = NULL;
+
+  if (!read_string (reader, &s, allow_blank))
+    return FALSE;
+
+  *write_to = g_intern_string (s);
+  g_free (s);
+  return TRUE;
+}
+
 static gboolean
 read_uint (xmlTextReaderPtr reader, guint * write_to)
 {
@@ -200,7 +220,8 @@ load_pad_template (xmlTextReaderPtr reader)
 {
   int ret;
   int depth = xmlTextReaderDepth (reader);
-  gchar *name = NULL, *caps_str = NULL;
+  const gchar *name = NULL;
+  gchar *caps_str = NULL;
   guint direction = 0, presence = 0;
 
   while ((ret = xmlTextReaderRead (reader)) == 1) {
@@ -208,7 +229,7 @@ load_pad_template (xmlTextReaderPtr reader)
       GstStaticPadTemplate *template;
 
       template = g_new0 (GstStaticPadTemplate, 1);
-      template->name_template = name;
+      template->name_template = name;   /* must be an interned string! */
       template->presence = presence;
       template->direction = direction;
       template->static_caps.string = caps_str;
@@ -220,7 +241,7 @@ load_pad_template (xmlTextReaderPtr reader)
       const gchar *tag = (gchar *) xmlTextReaderConstName (reader);
 
       if (g_str_equal (tag, "nametemplate")) {
-        read_string (reader, &name, FALSE);
+        read_const_interned_string (reader, &name, FALSE);
       } else if (g_str_equal (tag, "direction")) {
         read_enum (reader, GST_TYPE_PAD_DIRECTION, &direction);
       } else if (g_str_equal (tag, "presence")) {
@@ -230,7 +251,6 @@ load_pad_template (xmlTextReaderPtr reader)
       }
     }
   }
-  g_free (name);
   g_free (caps_str);
 
   return NULL;
@@ -241,21 +261,20 @@ load_feature (xmlTextReaderPtr reader)
 {
   int ret;
   int depth;
-  gchar *feature_name;
+  xmlChar *feature_name;
   GstPluginFeature *feature;
   GType type;
 
   depth = xmlTextReaderDepth (reader);
-  feature_name =
-      (gchar *) xmlTextReaderGetAttribute (reader, BAD_CAST "typename");
+  feature_name = xmlTextReaderGetAttribute (reader, BAD_CAST "typename");
 
-  GST_LOG ("loading feature");
+  GST_LOG ("loading feature '%s'", GST_STR_NULL ((const char *) feature_name));
 
   if (!feature_name)
     return NULL;
 
-  type = g_type_from_name (feature_name);
-  g_free (feature_name);
+  type = g_type_from_name ((const char *) feature_name);
+  xmlFree (feature_name);
   feature_name = NULL;
 
   if (!type) {
@@ -390,7 +409,7 @@ load_plugin (xmlTextReaderPtr reader, GList ** feature_list)
       if (g_str_equal (tag, "name")) {
         int ret;
 
-        ret = read_string (reader, &plugin->desc.name, FALSE);
+        ret = read_const_interned_string (reader, &plugin->desc.name, FALSE);
         GST_LOG ("name ret=%d, name=%s", ret, plugin->desc.name);
         if (!ret)
           break;
@@ -408,31 +427,31 @@ load_plugin (xmlTextReaderPtr reader, GList ** feature_list)
         GST_LOG ("filename %s", plugin->filename);
         plugin->basename = g_path_get_basename (plugin->filename);
       } else if (g_str_equal (tag, "version")) {
-        if (!read_string (reader, &plugin->desc.version, TRUE)) {
+        if (!read_const_interned_string (reader, &plugin->desc.version, TRUE)) {
           GST_WARNING ("version field was invalid in registry");
           break;
         }
         GST_LOG ("version %s", plugin->desc.version);
       } else if (g_str_equal (tag, "license")) {
-        if (!read_string (reader, &plugin->desc.license, TRUE)) {
+        if (!read_const_interned_string (reader, &plugin->desc.license, TRUE)) {
           GST_WARNING ("license field was invalid in registry");
           break;
         }
         GST_LOG ("license %s", plugin->desc.license);
       } else if (g_str_equal (tag, "source")) {
-        if (!read_string (reader, &plugin->desc.source, TRUE)) {
+        if (!read_const_interned_string (reader, &plugin->desc.source, TRUE)) {
           GST_WARNING ("source field was invalid in registry");
           break;
         }
         GST_LOG ("source %s", plugin->desc.source);
       } else if (g_str_equal (tag, "package")) {
-        if (!read_string (reader, &plugin->desc.package, TRUE)) {
+        if (!read_const_interned_string (reader, &plugin->desc.package, TRUE)) {
           GST_WARNING ("package field was invalid in registry");
           break;
         }
         GST_LOG ("package %s", plugin->desc.package);
       } else if (g_str_equal (tag, "origin")) {
-        if (!read_string (reader, &plugin->desc.origin, TRUE)) {
+        if (!read_const_interned_string (reader, &plugin->desc.origin, TRUE)) {
           GST_WARNING ("failed to read origin");
           break;
         }
@@ -459,7 +478,7 @@ load_plugin (xmlTextReaderPtr reader, GList ** feature_list)
         GstPluginFeature *feature = load_feature (reader);
 
         if (feature) {
-          feature->plugin_name = g_strdup (plugin->desc.name);
+          feature->plugin_name = plugin->desc.name;     /* interned string */
           *feature_list = g_list_prepend (*feature_list, feature);
         }
       } else {
@@ -585,8 +604,8 @@ gst_registry_xml_read_cache (GstRegistry * registry, const char *location)
  * Save
  */
 static gboolean
-gst_registry_save_escaped (GstRegistry * registry, char *prefix, char *tag,
-    char *value)
+gst_registry_save_escaped (GstRegistry * registry, const char *prefix,
+    const char *tag, const char *value)
 {
   gboolean ret = TRUE;
 
@@ -902,22 +921,44 @@ gst_registry_xml_write_cache (GstRegistry * registry, const char *location)
   if (!gst_registry_save (registry, "</GST-PluginRegistry>\n"))
     goto fail;
 
-  close (registry->cache_file);
+  /* check return value of close(), write errors may only get reported here */
+  if (close (registry->cache_file) < 0)
+    goto close_failed;
 
   if (g_file_test (tmp_location, G_FILE_TEST_EXISTS)) {
 #ifdef WIN32
     g_remove (location);
 #endif
-    g_rename (tmp_location, location);
+    if (g_rename (tmp_location, location) < 0)
+      goto rename_failed;
+  } else {
+    /* FIXME: shouldn't we return FALSE here? */
   }
 
   g_free (tmp_location);
-
+  GST_INFO ("Wrote XML registry cache");
   return TRUE;
 
+/* ERRORS */
 fail:
-  close (registry->cache_file);
-  g_free (tmp_location);
-
-  return FALSE;
+  {
+    (void) close (registry->cache_file);
+    /* fall through */
+  }
+fail_after_close:
+  {
+    g_remove (tmp_location);
+    g_free (tmp_location);
+    return FALSE;
+  }
+close_failed:
+  {
+    GST_ERROR ("close() failed: %s", g_strerror (errno));
+    goto fail_after_close;
+  }
+rename_failed:
+  {
+    GST_ERROR ("g_rename() failed: %s", g_strerror (errno));
+    goto fail_after_close;
+  }
 }
