@@ -78,11 +78,11 @@
 #include "gstcontrolsource.h"
 #include "gstinterpolationcontrolsource.h"
 
-#define GST_CAT_DEFAULT gst_controller_debug
+#define GST_CAT_DEFAULT controller_debug
 GST_DEBUG_CATEGORY_EXTERN (GST_CAT_DEFAULT);
 
 static GObjectClass *parent_class = NULL;
-GQuark __gst_controller_key;
+GQuark priv_gst_controller_key;
 
 /* property ids */
 enum
@@ -188,6 +188,7 @@ gst_controller_find_controlled_property (GstController * self,
 
   for (node = self->properties; node; node = g_list_next (node)) {
     prop = node->data;
+    /* FIXME: eventually use GQuark to speed it up */
     if (!strcmp (prop->name, name)) {
       return prop;
     }
@@ -220,7 +221,7 @@ gst_controller_new_valist (GObject * object, va_list var_args)
 
   GST_INFO ("setting up a new controller");
 
-  self = g_object_get_qdata (object, __gst_controller_key);
+  self = g_object_get_qdata (object, priv_gst_controller_key);
   /* create GstControlledProperty for each property */
   while ((name = va_arg (var_args, gchar *))) {
     /* test if this property isn't yet controlled */
@@ -232,7 +233,7 @@ gst_controller_new_valist (GObject * object, va_list var_args)
           self = g_object_new (GST_TYPE_CONTROLLER, NULL);
           self->object = g_object_ref (object);
           /* store the controller */
-          g_object_set_qdata (object, __gst_controller_key, self);
+          g_object_set_qdata (object, priv_gst_controller_key, self);
           ref_existing = FALSE;
         } else {
           /* only want one single _ref(), even for multiple properties */
@@ -281,7 +282,7 @@ gst_controller_new_list (GObject * object, GList * list)
 
   GST_INFO ("setting up a new controller");
 
-  self = g_object_get_qdata (object, __gst_controller_key);
+  self = g_object_get_qdata (object, priv_gst_controller_key);
   /* create GstControlledProperty for each property */
   for (node = list; node; node = g_list_next (node)) {
     name = (gchar *) node->data;
@@ -294,7 +295,7 @@ gst_controller_new_list (GObject * object, GList * list)
           self = g_object_new (GST_TYPE_CONTROLLER, NULL);
           self->object = g_object_ref (object);
           /* store the controller */
-          g_object_set_qdata (object, __gst_controller_key, self);
+          g_object_set_qdata (object, priv_gst_controller_key, self);
           ref_existing = FALSE;
         } else {
           /* only want one single _ref(), even for multiple properties */
@@ -603,7 +604,7 @@ gst_controller_get (GstController * self, gchar * property_name,
       res = gst_control_source_get_value (prop->csource, timestamp, val);
       if (!res) {
         g_free (val);
-        val = FALSE;
+        val = NULL;
       }
     } else {
       g_object_get_property (self->object, prop->name, val);
@@ -862,7 +863,7 @@ _gst_controller_dispose (GObject * object)
     }
 
     /* remove controller from object's qdata list */
-    g_object_set_qdata (self->object, __gst_controller_key, NULL);
+    g_object_set_qdata (self->object, priv_gst_controller_key, NULL);
     g_object_unref (self->object);
     self->object = NULL;
     g_mutex_unlock (self->lock);
@@ -909,7 +910,7 @@ _gst_controller_class_init (GstControllerClass * klass)
   gobject_class->dispose = _gst_controller_dispose;
   gobject_class->finalize = _gst_controller_finalize;
 
-  __gst_controller_key = g_quark_from_string ("gst::controller");
+  priv_gst_controller_key = g_quark_from_static_string ("gst::controller");
 
   /* register properties */
   g_object_class_install_property (gobject_class, PROP_CONTROL_RATE,
@@ -923,7 +924,7 @@ _gst_controller_class_init (GstControllerClass * klass)
 }
 
 GType
-gst_controller_get_type ()
+gst_controller_get_type (void)
 {
   static GType type = 0;
 
