@@ -20,6 +20,16 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#ifdef HAVE_VALGRIND
+# include <valgrind/valgrind.h>
+#else
+# define RUNNING_ON_VALGRIND FALSE
+#endif
+
 #include <gst/check/gstcheck.h>
 
 GST_START_TEST (test_caps)
@@ -39,6 +49,10 @@ GST_START_TEST (test_caps)
   fail_unless (GST_BUFFER_CAPS (buffer) == caps);
   ASSERT_CAPS_REFCOUNT (caps, "caps", 2);
 
+  fail_unless (gst_buffer_get_caps (buffer) == caps);
+  gst_caps_unref (caps);
+  ASSERT_CAPS_REFCOUNT (caps, "caps", 2);
+
   caps2 = gst_caps_from_string ("audio/x-raw-float");
   ASSERT_CAPS_REFCOUNT (caps2, "caps2", 1);
 
@@ -68,6 +82,11 @@ GST_START_TEST (test_subbuffer)
 
   buffer = gst_buffer_new_and_alloc (4);
   memset (GST_BUFFER_DATA (buffer), 0, 4);
+  /* set some metadata */
+  GST_BUFFER_TIMESTAMP (buffer) = 1;
+  GST_BUFFER_DURATION (buffer) = 2;
+  GST_BUFFER_OFFSET (buffer) = 3;
+  GST_BUFFER_OFFSET_END (buffer) = 4;
 
   sub = gst_buffer_create_sub (buffer, 1, 2);
   fail_if (sub == NULL, "create_sub of buffer returned NULL");
@@ -76,6 +95,48 @@ GST_START_TEST (test_subbuffer)
           2) == 0, "subbuffer contains the wrong data");
   ASSERT_BUFFER_REFCOUNT (buffer, "parent", 2);
   ASSERT_BUFFER_REFCOUNT (sub, "subbuffer", 1);
+  fail_unless (GST_BUFFER_TIMESTAMP (sub) == -1,
+      "subbuffer has wrong timestamp");
+  fail_unless (GST_BUFFER_DURATION (sub) == -1, "subbuffer has wrong duration");
+  fail_unless (GST_BUFFER_OFFSET (sub) == -1, "subbuffer has wrong offset");
+  fail_unless (GST_BUFFER_OFFSET_END (sub) == -1,
+      "subbuffer has wrong offset end");
+  gst_buffer_unref (sub);
+
+  /* create a subbuffer of size 0 */
+  sub = gst_buffer_create_sub (buffer, 1, 0);
+  fail_if (sub == NULL, "create_sub of buffer returned NULL");
+  fail_unless (GST_BUFFER_SIZE (sub) == 0, "subbuffer has wrong size");
+  fail_unless (memcmp (GST_BUFFER_DATA (buffer) + 1, GST_BUFFER_DATA (sub),
+          0) == 0, "subbuffer contains the wrong data");
+  ASSERT_BUFFER_REFCOUNT (buffer, "parent", 2);
+  ASSERT_BUFFER_REFCOUNT (sub, "subbuffer", 1);
+  gst_buffer_unref (sub);
+
+  /* test if metadata is coppied, not a complete buffer copy so only the
+   * timestamp and offset fields are copied. */
+  sub = gst_buffer_create_sub (buffer, 0, 1);
+  fail_if (sub == NULL, "create_sub of buffer returned NULL");
+  fail_unless (GST_BUFFER_SIZE (sub) == 1, "subbuffer has wrong size");
+  fail_unless (GST_BUFFER_TIMESTAMP (sub) == 1,
+      "subbuffer has wrong timestamp");
+  fail_unless (GST_BUFFER_OFFSET (sub) == 3, "subbuffer has wrong offset");
+  fail_unless (GST_BUFFER_DURATION (sub) == -1, "subbuffer has wrong duration");
+  fail_unless (GST_BUFFER_OFFSET_END (sub) == -1,
+      "subbuffer has wrong offset end");
+  gst_buffer_unref (sub);
+
+  /* test if metadata is coppied, a complete buffer is copied so all the timing
+   * fields should be copied. */
+  sub = gst_buffer_create_sub (buffer, 0, 4);
+  fail_if (sub == NULL, "create_sub of buffer returned NULL");
+  fail_unless (GST_BUFFER_SIZE (sub) == 4, "subbuffer has wrong size");
+  fail_unless (GST_BUFFER_TIMESTAMP (sub) == 1,
+      "subbuffer has wrong timestamp");
+  fail_unless (GST_BUFFER_DURATION (sub) == 2, "subbuffer has wrong duration");
+  fail_unless (GST_BUFFER_OFFSET (sub) == 3, "subbuffer has wrong offset");
+  fail_unless (GST_BUFFER_OFFSET_END (sub) == 4,
+      "subbuffer has wrong offset end");
 
   /* clean up */
   gst_buffer_unref (sub);
@@ -182,7 +243,7 @@ GST_START_TEST (test_span)
   gst_buffer_unref (span);
   ASSERT_BUFFER_REFCOUNT (buffer, "parent", 3);
 
-/* clean up */
+  /* clean up */
   gst_buffer_unref (sub1);
   gst_buffer_unref (sub2);
   gst_buffer_unref (buffer);
@@ -273,6 +334,9 @@ GST_START_TEST (test_metadata_writable)
   GstBuffer *buffer, *sub1;
 
   buffer = gst_buffer_new_and_alloc (4);
+  GST_BUFFER_FLAG_SET (buffer, GST_BUFFER_FLAG_DISCONT);
+  GST_BUFFER_FLAG_SET (buffer, GST_BUFFER_FLAG_DELTA_UNIT);
+
   /* Buffer with refcount 1 should have writable metadata */
   fail_unless (gst_buffer_is_metadata_writable (buffer) == TRUE);
 
@@ -292,6 +356,17 @@ GST_START_TEST (test_metadata_writable)
   ASSERT_BUFFER_REFCOUNT (buffer, "buffer", 2);
   fail_unless (gst_buffer_is_metadata_writable (buffer) == FALSE);
 
+  /* Check that make_metadata_writable() maintains the buffer flags */
+  fail_unless (GST_BUFFER_FLAG_IS_SET (sub1, GST_BUFFER_FLAG_DISCONT));
+  fail_unless (GST_BUFFER_FLAG_IS_SET (sub1, GST_BUFFER_FLAG_DELTA_UNIT));
+
+  /* Unset flags on writable buffer, then make sure they're still
+   * set on the original buffer */
+  GST_BUFFER_FLAG_UNSET (sub1, GST_BUFFER_FLAG_DISCONT);
+  GST_BUFFER_FLAG_UNSET (sub1, GST_BUFFER_FLAG_DELTA_UNIT);
+  fail_unless (GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_DISCONT));
+  fail_unless (GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_DELTA_UNIT));
+
   /* Drop the subbuffer and check that the metadata is now writable again */
   ASSERT_BUFFER_REFCOUNT (sub1, "sub1", 1);
   gst_buffer_unref (sub1);
@@ -299,10 +374,85 @@ GST_START_TEST (test_metadata_writable)
 
   ASSERT_BUFFER_REFCOUNT (buffer, "buffer", 1);
   gst_buffer_unref (buffer);
-} GST_END_TEST;
+}
 
-Suite *
-gst_test_suite (void)
+GST_END_TEST;
+
+GST_START_TEST (test_copy)
+{
+  GstBuffer *buffer, *copy;
+
+  buffer = gst_buffer_new_and_alloc (4);
+  ASSERT_BUFFER_REFCOUNT (buffer, "buffer", 1);
+
+  copy = gst_buffer_copy (buffer);
+  ASSERT_BUFFER_REFCOUNT (buffer, "buffer", 1);
+  ASSERT_BUFFER_REFCOUNT (copy, "copy", 1);
+  /* data must be copied and thus point to different memory */
+  fail_if (GST_BUFFER_DATA (buffer) == GST_BUFFER_DATA (copy));
+
+  gst_buffer_unref (copy);
+  gst_buffer_unref (buffer);
+
+  /* a 0-sized buffer has NULL data as per docs */
+  buffer = gst_buffer_new_and_alloc (0);
+  fail_unless (GST_BUFFER_DATA (buffer) == NULL);
+  fail_unless (GST_BUFFER_SIZE (buffer) == 0);
+
+  /* copying a 0-sized buffer should not crash and also set
+   * the data member NULL. */
+  copy = gst_buffer_copy (buffer);
+  fail_unless (GST_BUFFER_DATA (copy) == NULL);
+  fail_unless (GST_BUFFER_SIZE (copy) == 0);
+
+  gst_buffer_unref (copy);
+  gst_buffer_unref (buffer);
+}
+
+GST_END_TEST;
+
+GST_START_TEST (test_try_new_and_alloc)
+{
+  GstBuffer *buf;
+
+  /* special case: alloc of 0 bytes results in new buffer with NULL data */
+  buf = gst_buffer_try_new_and_alloc (0);
+  fail_unless (buf != NULL);
+  fail_unless (GST_IS_BUFFER (buf));
+  fail_unless (GST_BUFFER_SIZE (buf) == 0);
+  fail_unless (GST_BUFFER_DATA (buf) == NULL);
+  fail_unless (GST_BUFFER_MALLOCDATA (buf) == NULL);
+  gst_buffer_unref (buf);
+
+  /* normal alloc should still work */
+  buf = gst_buffer_try_new_and_alloc (640 * 480 * 4);
+  fail_unless (buf != NULL);
+  fail_unless (GST_IS_BUFFER (buf));
+  fail_unless (GST_BUFFER_SIZE (buf) == (640 * 480 * 4));
+  fail_unless (GST_BUFFER_DATA (buf) != NULL);
+  fail_unless (GST_BUFFER_MALLOCDATA (buf) != NULL);
+  GST_BUFFER_DATA (buf)[640 * 479 * 4 + 479] = 0xff;
+  gst_buffer_unref (buf);
+
+#if 0
+  /* Disabled this part of the test, because it happily succeeds on 64-bit
+   * machines that have enough memory+swap, because the address space is large
+   * enough. There's not really any way to test the failure case except by 
+   * allocating chunks of memory until it fails, which would suck. */
+
+  /* now this better fail (don't run in valgrind, it will abort
+   * or warn when passing silly arguments to malloc) */
+  if (!RUNNING_ON_VALGRIND) {
+    buf = gst_buffer_try_new_and_alloc ((guint) - 1);
+    fail_unless (buf == NULL);
+  }
+#endif
+}
+
+GST_END_TEST;
+
+static Suite *
+gst_buffer_suite (void)
 {
   Suite *s = suite_create ("GstBuffer");
   TCase *tc_chain = tcase_create ("general");
@@ -315,22 +465,10 @@ gst_test_suite (void)
   tcase_add_test (tc_chain, test_is_span_fast);
   tcase_add_test (tc_chain, test_span);
   tcase_add_test (tc_chain, test_metadata_writable);
+  tcase_add_test (tc_chain, test_copy);
+  tcase_add_test (tc_chain, test_try_new_and_alloc);
+
   return s;
 }
 
-int
-main (int argc, char **argv)
-{
-  int nf;
-
-  Suite *s = gst_test_suite ();
-  SRunner *sr = srunner_create (s);
-
-  gst_check_init (&argc, &argv);
-
-  srunner_run_all (sr, CK_NORMAL);
-  nf = srunner_ntests_failed (sr);
-  srunner_free (sr);
-
-  return nf;
-}
+GST_CHECK_MAIN (gst_buffer);
