@@ -44,9 +44,9 @@ GST_START_TEST (test_from_string_int)
     0xFFFF,
     0xFFFF,
     0x7FFFFFFF,
-    0x80000000,
-    0xFF000000,
-    0xFF000000,
+    (gint) 0x80000000,
+    (gint) 0xFF000000,
+    (gint) 0xFF000000,
   };
   GstStructure *structure;
   int i;
@@ -115,6 +115,24 @@ GST_START_TEST (test_from_string)
   fail_unless (G_VALUE_HOLDS_BOOLEAN (val));
   fail_unless_equals_int (g_value_get_boolean (val), TRUE);
   gst_structure_free (structure);
+
+  /* This should still work for now (FIXME: 0.11) */
+  s = "0.10:decoder-video/mpeg, abc=(boolean)false";
+  structure = gst_structure_from_string (s, NULL);
+  fail_if (structure == NULL, "Could not get structure from string %s", s);
+  gst_structure_free (structure);
+
+  /* make sure we bail out correctly in case of an error or if parsing fails */
+  s = "***foo***, abc=(boolean)false";
+  structure = gst_structure_from_string (s, NULL);
+  fail_unless (structure == NULL);
+
+  /* assert that we get a warning if the structure wasn't entirely consumed, but
+   * we didn't provide an end pointer */
+  s = "foo/bar; other random data";
+  ASSERT_WARNING (structure = gst_structure_from_string (s, NULL));
+  fail_if (structure == NULL, "Could not get structure from string %s", s);
+  gst_structure_free (structure);
 }
 
 GST_END_TEST;
@@ -131,14 +149,18 @@ GST_START_TEST (test_to_string)
 #if 0
   ASSERT_CRITICAL (st1 = gst_structure_new ("Foo with whitespace", NULL));
   fail_unless (st1 == NULL);
+  ASSERT_CRITICAL (st1 = gst_structure_new ("1st", NULL));
+  fail_unless (st1 == NULL);
 #else
   st1 = gst_structure_new ("Foo with whitespace is still allowed", NULL);
   fail_unless (st1 != NULL);
   gst_structure_free (st1);
-#endif
 
-  ASSERT_CRITICAL (st1 = gst_structure_new ("1st", NULL));
-  fail_unless (st1 == NULL);
+  /* structure names starting with a number are also still allowed */
+  st1 = gst_structure_new ("1st", NULL);
+  fail_unless (st1 != NULL);
+  gst_structure_free (st1);
+#endif
 }
 
 GST_END_TEST;
@@ -234,11 +256,19 @@ GST_START_TEST (test_structure_new)
 
   gst_structure_free (s);
 
-  domain = g_quark_from_string ("test");
+  domain = g_quark_from_static_string ("test");
   e = g_error_new (domain, 0, "a test error");
   s = gst_structure_new ("name", "key", GST_TYPE_G_ERROR, e, NULL);
   g_error_free (e);
   gst_structure_free (s);
+
+  /* This should still work for now (FIXME 0.11) */
+  gst_structure_free (gst_structure_new ("0.10:decoder-video/mpeg", NULL));
+
+  /* make sure we bail out correctly in case of an error or if parsing fails */
+  ASSERT_CRITICAL (s = gst_structure_new ("^joo\nba\ndoo^",
+          "abc", G_TYPE_BOOLEAN, FALSE, NULL));
+  fail_unless (s == NULL);
 }
 
 GST_END_TEST;
@@ -342,7 +372,7 @@ GST_START_TEST (test_structure_nested)
 
 GST_END_TEST;
 
-Suite *
+static Suite *
 gst_structure_suite (void)
 {
   Suite *s = suite_create ("GstStructure");
